@@ -167,19 +167,21 @@ class Archive(object):
         """
         if PY2:
             return self._tagged.iterkeys()
-        return self._tagged.keys()
+        else:
+            return self._tagged.keys()
 
     def values(self):
         """Return a list of uncertain numbers 
         """
-        return self._tagged.values()
+        return self._tagged.values() 
         
     def itervalues(self):
         """Return an iterator for uncertain numbers 
         """
         if PY2:
             return self._tagged.itervalues()
-        return self._tagged.values()
+        else:
+            return self._tagged.values()
 
     def items(self):
         """Return a list of name -to- uncertain-number pairs 
@@ -191,7 +193,8 @@ class Archive(object):
         """
         if PY2:
             return self._tagged.iteritems()
-        return self._tagged.items()
+        else:
+            return self._tagged.items()
 
     def __len__(self):
         """Return the number of entries 
@@ -214,7 +217,14 @@ class Archive(object):
                     )
                     
                 if not value.is_elementary:
-                    self._uid_to_intermediate[value.real._node.uid] = value.real
+                    try:
+                        uid = value.real._node.uid
+                    except AttributeError:
+                        raise RuntimeError(
+                            "uncertain number labelled '{}' is not declared intermediate".format(key)
+                        )
+                        
+                    self._uid_to_intermediate[uid] = value.real            
                 
                 self._tagged_reals[key] = value
 
@@ -236,8 +246,16 @@ class Archive(object):
                 self._tagged_reals[n_im] = value.imag
                 
                 if not value.is_elementary:
-                    self._uid_to_intermediate[value.real._node.uid] = value.real
-                    self._uid_to_intermediate[value.imag._node.uid] = value.imag
+                    try:
+                        uid_r = value.real._node.uid
+                        uid_i = value.imag._node.uid
+                    except AttributeError:
+                        raise RuntimeError(
+                            "uncertain number labelled '{}' is not declared intermediate".format(key)
+                        )
+                        
+                    self._uid_to_intermediate[uid_r] = value.real
+                    self._uid_to_intermediate[uid_i] = value.imag
                         
             else:
                 raise RuntimeError(
@@ -277,8 +295,12 @@ class Archive(object):
             >>> a = pr.Archive()
             >>> x = ureal(1,1)
             >>> y = ureal(2,1)
+            >>> z = ureal(20,1)
             >>> a.add(x=x,fred=y)
 
+            # Entries can also be added using the name as a key       
+            >>> a['z'] = z
+
         .. invisible-code-block: pycon
         
             >>> import tempfile
@@ -290,7 +312,8 @@ class Archive(object):
         
             >>> pr.dump(f, a)
             >>> f.close()
-        
+   
+
         """
         if self._extract:
             raise RuntimeError('This archive is write-only!')
@@ -345,18 +368,22 @@ class Archive(object):
         
             >>> a = pr.load(f)
             >>> f.close()
-            >>>
+            
             >>> a.extract('fred')
             ureal(2.0,1.0,inf)
             >>> x, fred = a.extract('x','fred')
             >>> x
             ureal(1.0,1.0,inf)            
-        
+ 
+            # Entries can also be extracted using the name as a key
+            >>> a['z']
+            ureal(20.0,1.0,inf)
+
         .. invisible-code-block: pycon
             
             >>> import os, tempfile
             >>> os.remove(tempfile.gettempdir() + '/GTC-archive-test.gar')
-            
+  
         """        
         if not self._extract:
             raise RuntimeError('This archive is read-only!')
@@ -371,8 +398,14 @@ class Archive(object):
         
         NB after freezing, the archive object is immutable.
         
-        """        
+        """    
+        if not len(self):
+            raise RuntimeError(
+                "There is nothing in the archive!"
+            )
+
         values = self._tagged_reals.itervalues() if PY2 else self._tagged_reals.values()
+        
         self._leaf_nodes = {
             n_i.uid  : LeafNode(n_i)
                 for un in values
@@ -506,6 +539,7 @@ class Archive(object):
     def _thaw(self):
         _leaf_nodes = dict()
         items = self._leaf_nodes.iteritems() if PY2 else self._leaf_nodes.items()
+                        
         for uid_i,fl_i in items:
             l = context._context.new_leaf(
                 uid_i, 
@@ -552,9 +586,8 @@ class Archive(object):
                 )                    
                 self._tagged[name] = un
                 
-            elif isinstance(
-                    obj,Complex
-                ):
+            elif isinstance(obj,Complex):
+                # This is an intermediate uncertain complex
                 name_re = obj.n_re
                 name_im = obj.n_im
                 
@@ -570,8 +603,18 @@ class Archive(object):
                 )
 
                 assert un_re.is_elementary == un_im.is_elementary
+                
                 unc = UncertainComplex(un_re,un_im)
+                
+                # An intermediate complex needs to 
+                # link the nodes of its components 
+                # (same as in `UncertainComplex._intermediate`)
+                complex_id = (un_re._node.uid,un_im._node.uid)
+                unc.real._node.complex = complex_id 
+                unc.imag._node.complex = complex_id
+        
                 self._tagged[name] = unc
+                
             else:
                 assert False
                         
